import { Updater } from './utils';
import { ArrayQueryKey, IsFetchingMoreValue, QueryConfig, QueryStatus } from './types';
import { QueryCache } from './queryCache';
import { QueryObserver, UpdateListener } from './queryObserver';
interface QueryInitConfig<TResult, TError> {
    queryCache: QueryCache;
    queryKey: ArrayQueryKey;
    queryHash: string;
    config: QueryConfig<TResult, TError>;
    notifyGlobalListeners: (query: Query<TResult, TError>) => void;
}
export interface QueryState<TResult, TError> {
    canFetchMore?: boolean;
    data?: TResult;
    error: TError | null;
    failureCount: number;
    isError: boolean;
    isFetched: boolean;
    isFetching: boolean;
    isFetchingMore: IsFetchingMoreValue;
    isIdle: boolean;
    isLoading: boolean;
    isSuccess: boolean;
    status: QueryStatus;
    throwInErrorBoundary?: boolean;
    updatedAt: number;
}
interface FetchOptions {
    fetchMore?: FetchMoreOptions;
}
export interface FetchMoreOptions {
    fetchMoreVariable?: unknown;
    previous: boolean;
}
export interface RefetchOptions {
    throwOnError?: boolean;
}
export declare enum ActionType {
    Failed = "Failed",
    Fetch = "Fetch",
    Success = "Success",
    Error = "Error"
}
interface FailedAction {
    type: ActionType.Failed;
}
interface FetchAction {
    type: ActionType.Fetch;
    isFetchingMore?: IsFetchingMoreValue;
}
interface SuccessAction<TResult> {
    type: ActionType.Success;
    data: TResult | undefined;
    canFetchMore?: boolean;
}
interface ErrorAction<TError> {
    type: ActionType.Error;
    error: TError;
}
export declare type Action<TResult, TError> = ErrorAction<TError> | FailedAction | FetchAction | SuccessAction<TResult>;
export declare class Query<TResult, TError> {
    queryKey: ArrayQueryKey;
    queryHash: string;
    config: QueryConfig<TResult, TError>;
    observers: QueryObserver<TResult, TError>[];
    state: QueryState<TResult, TError>;
    cacheTime: number;
    private queryCache;
    private promise?;
    private gcTimeout?;
    private cancelFetch?;
    private continueFetch?;
    private isTransportCancelable?;
    private notifyGlobalListeners;
    constructor(init: QueryInitConfig<TResult, TError>);
    private updateConfig;
    private dispatch;
    private scheduleGc;
    cancel(): void;
    private continue;
    private clearTimersObservers;
    private clearGcTimeout;
    setData(updater: Updater<TResult | undefined, TResult>): void;
    clear(): void;
    destroy(): void;
    isEnabled(): boolean;
    isStale(): boolean;
    isStaleByTime(staleTime?: number): boolean;
    onWindowFocus(): Promise<void>;
    onOnline(): Promise<void>;
    subscribe(listener?: UpdateListener<TResult, TError>): QueryObserver<TResult, TError>;
    subscribeObserver(observer: QueryObserver<TResult, TError>): void;
    unsubscribeObserver(observer: QueryObserver<TResult, TError>): void;
    refetch(options?: RefetchOptions, config?: QueryConfig<TResult, TError>): Promise<TResult | undefined>;
    fetchMore(fetchMoreVariable?: unknown, options?: FetchMoreOptions, config?: QueryConfig<TResult, TError>): Promise<TResult | undefined>;
    fetch(options?: FetchOptions, config?: QueryConfig<TResult, TError>): Promise<TResult | undefined>;
    private startFetch;
    private startInfiniteFetch;
    private tryFetchData;
}
export declare function queryReducer<TResult, TError>(state: QueryState<TResult, TError>, action: Action<TResult, TError>): QueryState<TResult, TError>;
export {};
